# Demokritos Atom Library and Server
# Copyright (C) 2005-2006 James Tauber
# See GPL file at top of distribution

"""
This module models the objects in the Atom Publication Protocol (APP).

The objects are designed to be usable on both the server and client side.

One implication of this is that a Workspace object doesn't actually contain
Collection objects but rather just CollectionReference objects which contain
enough to produce the introspection document.

Likewise, a Member object only contains enough content to produce the
collection feed. Any out-of-line content is referenced but not included.

As can be seen in store.py, an actual server implementation has to extend these
objects to provide full APP functionality.
"""

from atom import Entry


class Extensible:
    """
    base class of an object that can have extension elements.
    """
    
    def __init__(self):
        self.children = []

    def add(self, child):
        self.children.append(child)

        
class Service(Extensible):

    # workspaces[]
    
    def __init__(self):
        Extensible.__init__(self)
        self.workspaces = []
    
    
class Workspace(Extensible):

    # title
    # collections[] - the first collection is the primary one (APP-07 7.2.2)

    def __init__(self, title=None):
        Extensible.__init__(self)
        if title:
            self.title = title
            
        self.collections = []

    def add_collection(self, title, href, member_type):
        self.collections.append(CollectionReference(title, href, member_type))

    
class CollectionReference(Extensible):

    # title
    # href
    # member_type

    def __init__(self, title=None, href=None, member_type=None):
        # note that if arguments are not provided here, the attributes must
        # be set individually.

        Extensible.__init__(self)

        self.title = title
        self.href = href
        self.member_type = member_type    


def updated_sort_key(member):
    return member.updated.value # @@@ need to normalize timezone


class Collection:

    # members[]
    
    def __init__(self):
        self.members = []
    
    def add_member(self, member):
        self._remove_member(member)
        self.members.insert(0, member)
        self.members.sort(key=updated_sort_key, reverse=True)

    # underscore variant exists so add_member above doesn't call version
    # on subclass
    def _remove_member(self, member):
        for m in self.members[:]:
            if m.id == member.id:
                self.members.remove(m)

    def remove_member(self, member):
        self._remove_member(member)


class Member:
    pass
