# Demokritos Atom Library and Server
# Copyright (C) 2006 James Tauber
# See GPL file at top of distribution

import cPickle as pickle

from pyworks.svn import SvnRepository, SvnNode

from store import ServerEntryCollection, ServerMediaCollection
from store import ServerEntryMember, ServerMediaMember

NONE = 0
COLLECTION = 1
MEMBER = 2
OTHER = 3

class Persistence:

    def __init__(self, repos_path, store):
        self.store = store
        self.svn_repository = SvnRepository(repos_path)
         
    def get_type(self, path):
        node = self.svn_repository.get_node(path)
        if node.node_kind() == SvnNode.NONE:
            return NONE
        if node.node_kind() == SvnNode.DIR:
            return OTHER
        # @@@ hack - could use dedicated property
        member_type = node.get_property("app:member-type")
        if member_type:
            return COLLECTION
        else:
            return MEMBER

    def get_collection(self, path):
        node = self.svn_repository.get_node(path)
        if node.node_kind() == SvnNode.NONE:
            return None
        else:
            if node.get_property("app:member-type") == "entry":
                collection = PersistedEntryCollection(path, node)
            else:
                collection = PersistedMediaCollection(path, node)
            collection.dereference_members(self)
            collection.store = self.store
            return collection

    def get_member(self, path):
        node = self.svn_repository.get_node(path)
        if node.node_kind() == SvnNode.NONE:
            return None
        else:
            if node.get_property("app:media-type") == "application/atom+xml":
                member = PersistedEntryMember(path, node)
            else:
                member = PersistedMediaMember(path, node)
            return member

    def remove_member(self, member):
        collection = self.get_collection(member.collection_path)
        collection.remove_member(member)
        txn = self.svn_repository.begin_txn()
        txn.delete(member.path)
        txn.commit()
        # @@@ could be done in same transaction
        self.save_collection(collection)

    def save_member(self, member, collection):
        node = self.svn_repository.get_node(member.path)
        txn = self.svn_repository.begin_txn()
        if node.node_kind() == SvnNode.NONE:
            txn.make_dirs(member.path)
            txn.make_file(member.path)
        txn.set_content(member.path, member.get_content())
        txn.change_node_prop(member.path, "svn:mime-type", member.media_type)
        txn.change_node_prop(member.path, "atom:href", str(member.href))
        txn.change_node_prop(member.path, "atom:id", str(member.id))
        txn.change_node_prop(member.path, "app:collection", str(member.collection_path))
        txn.change_node_prop(member.path, "atom:title", pickle.dumps(member.title))
        txn.change_node_prop(member.path, "atom:updated", pickle.dumps(member.updated))
        txn.change_node_prop(member.path, "atom:authors", pickle.dumps(member.authors))
        # @@@ other properties
        txn.commit()

    def save_collection(self, collection):
        node = self.svn_repository.get_node(collection.path)
        txn = self.svn_repository.begin_txn()
        if node.node_kind() == SvnNode.NONE:
            txn.make_dirs(collection.path)
            txn.make_file(collection.path)
        member_path_list = "\n".join(collection.member_path_list)
        txn.set_content(collection.path, str(member_path_list))
        txn.change_node_prop(collection.path, "app:member-type", str(collection.member_type))
        txn.change_node_prop(collection.path, "atom:id", str(collection.id))
        txn.change_node_prop(collection.path, "atom:title", str(collection.title))
        txn.change_node_prop(collection.path, "atom:updated", pickle.dumps(collection.updated))
        txn.change_node_prop(collection.path, "atom:href", str(collection.href))
        txn.commit()


class PersistedCollection:

    def __init__(self, path, svn_node):
        self.path = path
        self.svn_node = svn_node
        self.member_type = svn_node.get_property("app:member-type")
        self.id = svn_node.get_property("atom:id")
        self.title = svn_node.get_property("atom:title")
        updated_pickle = svn_node.get_property("atom:updated")
        if updated_pickle:
            self.updated = pickle.loads(updated_pickle)
        self.href = svn_node.get_property("atom:href")

        content = self.svn_node.get_content()

        if content == "":
            self.member_path_list = []
        else:
            self.member_path_list = content.split("\n")

    def dereference_members(self, persistence):
        self.members = [persistence.get_member(path) for path in self.member_path_list]


class PersistedEntryCollection(PersistedCollection, ServerEntryCollection):
    pass
    
class PersistedMediaCollection(PersistedCollection, ServerMediaCollection):
    pass


class PersistedMember:

    def __init__(self, path, svn_node):
        self.path = path
        self.svn_node = svn_node
        self.media_type = svn_node.get_property("svn:mime-type")
        self.href = svn_node.get_property("atom:href")
        self.id = svn_node.get_property("atom:id")
        self.collection_path = svn_node.get_property("app:collection")
        title_pickle = svn_node.get_property("atom:title")
        if title_pickle:
            self.title = pickle.loads(title_pickle)
        updated_pickle = svn_node.get_property("atom:updated")
        if updated_pickle:
            self.updated = pickle.loads(updated_pickle)
        authors_pickle = svn_node.get_property("atom:authors")
        if authors_pickle:
            self.authors = pickle.loads(authors_pickle)
        # @@@ other properties
        
    def get_content(self):
        return self.svn_node.get_content()


class PersistedMediaMember(PersistedMember, ServerMediaMember):
    pass
    

class PersistedEntryMember(PersistedMember, ServerEntryMember):
    pass
    
