# pyworks.svn - object-oriented wrapper around subversion binding
# Copyright (C) 2006 James Tauber
# See GPL file at top of distribution

"""
This module provides a more object-oriented wrapper around the SWIG-based
binding that comes with the version control system, Subversion.

The main purpose of this module is to enable use of a local subversion
repository for persistence.

Subversion 1.3.0 or later is required.
"""

from svn import fs, repos, core

assert (core.SVN_VER_MAJOR, core.SVN_VER_MINOR) >= (1, 3), "Subversion 1.3 or later required"

class SvnRepository:
    """
    A wrapper around a subversion repository and filesystem.
    """

    def __init__(self, root_path):
        self.repos = repos.open(root_path)
        self.fs_ptr = repos.fs(self.repos)

    def latest_rev(self):
        return fs.youngest_rev(self.fs_ptr)

    def get_node(self, path, revision=None):
        revision = revision or self.latest_rev()
        revision_root = fs.revision_root(self.fs_ptr, revision)
        return SvnNode(self.fs_ptr, revision_root, path)
    
    def get_revision_property(self, name, revision=None):
        revision = revision or self.latest_rev()
        return fs.revision_prop(self.fs_ptr, revision, name)

    def begin_txn(self, revision=None):
        revision = revision or self.latest_rev()
        return SvnTransaction(fs.begin_txn(self.fs_ptr, revision))


class SvnTransaction:
    """
    A wrapper around a subversion transaction and transaction root.
    """

    def __init__(self, txn):
        self.txn = txn
        self.root = fs.txn_root(txn)

    def commit(self):
        return fs.commit_txn(self.txn)

    def abort(self):
        return fs.abort_txn(self.txn)

    def make_dirs(self, path):
        parts = path.split("/")
        for i in range(len(parts)):
            d = "/".join(parts[:i])
            if d:
                if fs.check_path(self.root, d) == SvnNode.NONE:
                    self.make_dir(d)

    def make_dir(self, path):
        fs.make_dir(self.root, path)

    def make_file(self, path):
        fs.make_file(self.root, path)
        
    def change_node_prop(self, path, name, value):
        fs.change_node_prop(self.root, path, name, value)

    def delete(self, path):
        fs.delete(self.root, path)

    def set_content(self, path, content):
        stream = fs.apply_text(self.root, path, None)
        core.svn_stream_write(stream, content)
        core.svn_stream_close(stream)


class SvnNode:
    """
    A subversion node (revision root / path pair).
    """

    def __init__(self, fs_ptr, revision_root, path):
        self.fs_ptr = fs_ptr
        self.root = revision_root
        self.path = path

    # node kinds
    NONE, FILE, DIR, UNKNOWN = range(4)

    def node_kind(self):
        return fs.check_path(self.root, self.path)
        
    def get_stream(self):
        stream = fs.file_contents(self.root, self.path)
        return core.Stream(stream)

    def get_content(self):
        return self.get_stream().read()

    def get_entries(self):
        # @@@ could return nodes
        return fs.dir_entries(self.root, self.path).keys()

    def get_property(self, name):
        return fs.node_prop(self.root, self.path, name)

    def get_properties(self):
        return fs.node_proplist(self.root, self.path)

    def last_modified(self):
        created_rev = fs.node_created_rev(self.root, self.path)
        date = fs.revision_prop(self.fs_ptr, created_rev, "svn:date")
        return date
#        return core.svn_time_from_cstring(date) / 1000000
