# pyworks.web - http server with more RESTful approach
# Copyright (C) 2005-2006 James Tauber
# See GPL file at top of distribution

"""
This module provides a base for an HTTP server that encourages a RESTful
resource-oriented approach.

Server should be instantiated with a handler that subclasses RequestHandler.
"""

import traceback
import BaseHTTPServer
import urllib
import cgi


class Server:
    """
    instantiate or subclass this and pass a subclass of RequestHandler to
    __init__ as the handler.
    """

    def __init__(self, host, port, handler):
        self.httpd = BaseHTTPServer.HTTPServer((host, port), handler)
        
    def serve_forever(self):
        self.httpd.serve_forever()


class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    """
    handles HTTP requests.
    
    subclass this and:
     - set server_version
     - implement a get_resource(self, request) method which returns the
       appropriate Resource subclass for the given request.
    """
    
    ## server_version = "..."
    
    ## def get_resource(self, request):
    ##     ...return appropriate Resource based on request...
    
    def do_HEAD(self):      self._handle("HEAD")
    def do_GET(self):       self._handle("GET")
    def do_POST(self):      self._handle("POST")
    def do_PUT(self):       self._handle("PUT")
    def do_DELETE(self):    self._handle("DELETE")
        
    def _handle(self, method):
        try:
            request = Request(self)
            resource = self.get_resource(request)
            resource.request = request
            if not hasattr(resource, method):
                self.send_error(405, "Cannot %s this URI" % method)
            else:
                response = getattr(resource, method)()
                response.output(self)
        except HTTPError, e:
            code, message = e
            self.send_error(code, message)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")


class Request:
    """
    encapsulates an HTTP request.
    
    fields:
      full_path - full path including query string
      path      - path without query string
      header    - dictionary of headers
      content   - body content (if any)
    """
    
    def __init__(self, request_handler):
        self.full_path = request_handler.path
        self.path, self.query = urllib.splitquery(self.full_path)
        self.header = request_handler.headers
        length = request_handler.headers.get("Content-Length")
        if length:
            self.content = request_handler.rfile.read(int(length))
        # @@@ only read content if there is a Content-Length header
        # @@@ seems to hang otherwise - need to investigate
        # @@@ could return a 411 in the mean-time

    def parse(self, types):
        """
        parses the query string according to the required types, raising
        a ValueError if the query string doesn't conform.
        
        Currently only supports a single field. types should be a tuple of
        field_name and field_type where field_type must be able to take
        a string to generate the desired object, e.g.
        
            index = self.request.parse(("index", int))
        
        will return an integer index if it the sole parameter in the
        query string otherwise a ValueError will be raised.
        """
        
        if isinstance(types, list):
             pass # @@@ to implement
        else:
            field_name, field_type = types
            qsl = cgi.parse_qsl(self.query, strict_parsing=True)
            if len(qsl) != 1:
                raise ValueError
            if qsl[0][0] != field_name:
                raise ValueError
            field_value = field_type(qsl[0][1])
        return field_value


class Response:
    """
    encapsulates the response to be given by the HTTP server.
    """

    def __init__(self, code, header={}, content=None):
        self.code = code
        self.header = header
        self.content = content
        
    def output(self, handler):
        handler.send_response(self.code)
        for header in self.header:
            handler.send_header(header, self.header[header])
        handler.end_headers()
        if self.content:
            handler.wfile.write(self.content)


class Resource:
    """
    an object identified by a URI (looked up by get_resource on RequestHandler)
    that responds to HTTP methods such as HEAD, GET, POST, PUT, DELETE.

    For convenience, HEAD and GET are already implemented (you'll just need
    to implement a content() method---see below). You may also implement one 
    or more of POST, PUT, DELETE.

    The HEAD, GET, POST, PUT, DELETE methods of a Resource should either
    return one of these objects, e.g.

        return Response(200, {"Content-type": media_type}, content)
        
        return Response(201, {"Location": new_uri})

    or raise an HTTPError:
    
        raise HTTPError(415, "Unsupported Media Type")        

        raise HTTPError(404, "Resource Not Found")    
    
    As well as passing the parts of the response in the constructor, you
    can also build them up individually, e.g.
    
        response = Response(200)
        response.header["Content-type"] = content_type
        response.header["Content-Length"] = str(len(content))
        response.content = content
        return response

    Note that you should only implement those HTTP methods that are appropriate.
    The server will automatically return a 405 if a client uses a method your
    resource doesn't support.
    
    While you can implement your own HEAD and GET methods, it is easier to
    use the defaults and instead implement a content() method which either
    returns a tuple of strings (content_type, content) or raises an HTTPError.
    """    

    def HEAD(self):
        content_type, content = self.content()
        response = Response(200)
        response.header["Content-type"] = content_type
        response.header["Content-Length"] = str(len(content))
        return response

    def GET(self):
        content_type, content = self.content()
        response = Response(200)
        response.header["Content-type"] = content_type
        response.header["Content-Length"] = str(len(content))
        response.content = content
        return response

    ## def content(self):
    ##     ...return content_type, content
    
    ## implement POST, PUT, DELETE if necessary


class HTTPError(Exception):
    """
    raise this for 4xx HTTP errors.
    
    You should pass in two args: the HTTP code (as an int) and an error message
    string, e.g.

        raise HTTPError(404, "Resource Not Found")

    """
    
    pass
