# Demokritos Atom Library and Server
# Copyright (C) 2005-2006 James Tauber
# See GPL file at top of distribution

"""
The Atom HTTP server, built on webbase that provides the link between HTTP 
requests and the store itself.
"""

from cStringIO import StringIO
import time

from store import UnsupportedMediaTypeError
from app import generate_introspection
from atom import generate_feed, generate_entry, AtomException

from pyworks.web import Server, RequestHandler, Response, Resource, HTTPError


## Note that this module gets injected with the atom_store object.


class AtomServer(Server):

    def __init__(self, host="", port=8000):
        Server.__init__(self, host, port, AtomRequestHandler)
        print "listening on port", port
        

class AtomRequestHandler(RequestHandler):
    
    server_version = "Demokritos/0.3"
    
    def get_resource(self, request):
        if request.full_path == "/introspection":
            return IntrospectionResource()
        elif atom_store.is_collection(request.path):
            collection = atom_store.get_collection(request.path)
            return CollectionResource(collection)
        elif atom_store.is_member(request.path):
            member = atom_store.get_member(request.path)
            return MemberResource(member)
        else:
            raise HTTPError(404, "Resource Not Found")


class IntrospectionResource(Resource):
    """
    the Resource representing introspection documents.
    """

    def content(self):
        content = StringIO()
        generate_introspection(content, atom_store.service)
        return "application/atomserv+xml", content.getvalue()


class CollectionResource(Resource):
    """
    the Resource representing collection feeds.
    """

    def __init__(self, collection):
        self.collection = collection
        
    def content(self):
        if self.collection:
            if self.request.query:
                try:
                    index = self.request.parse(("index", int))
                except ValueError: # @@@ catches too much
                    raise HTTPError(400, "Bad Query String")
            else:
                index = 0
            
            try:
                feed = self.collection.make_feed(index)
            except AttributeError: # @@@ catches too much
                raise HTTPError(400, "Bad Query String")
        
            content = StringIO()
            generate_feed(content, feed)
            return "application/atom+xml", content.getvalue()
        else:
            raise HTTPError(404, "Resource Not Found")
    
    def POST(self):
        # add new resource to collection and return 201 with location

        if self.collection:
            suggested_title = self.request.header.get("Title")
            media_type = self.request.header.get("Content-type")
            content = self.request.content

            try:
                new_member = self.collection.create_member(content, media_type, suggested_title)
                entry = new_member.make_feed_entry()
                entry_content = StringIO()
                generate_entry(entry_content, entry, top=True)

                return Response(201, {"Location": new_member.href}, entry_content.getvalue())

            except AtomException, e:
                raise HTTPError(400, e)
            except UnsupportedMediaTypeError:
                raise HTTPError(415, "Unsupported Media Type")
        else:
            raise HTTPError(404, "Collection Not Found")

    ## Note you cannot PUT or DELETE a CollectionResource


class MemberResource(Resource):
    """
    the Resource representing individual members of a collection.
    """

    def __init__(self, member):
        self.member = member

    def content(self):
        if self.member:
            if self.request.query:
                if self.request.query == "atom":
                    entry = self.member.make_feed_entry()
                    content = StringIO()
                    generate_entry(content, entry, top=True)
                    return "application/atom+xml", content.getvalue()
                else:
                    raise HTTPError(400, "Bad Query String")
            else:
                return self.member.media_type, self.member.get_content()
        else:
            raise HTTPError(404, "Resource Not Found")

    ## Note you cannot POST to a MemberResource

    def PUT(self):
        # replace resource, return 200 with resource

        if self.member:
            media_type = self.request.header.get("Content-type")
            suggested_title = self.request.header.get("Title")
    
            content = self.request.content
        
            self.member = atom_store.replace_member(self.member, content, media_type, suggested_title)
    
            # @@@ this could also return a 204 with no content
            return Response(200, {"Content-type": self.member.media_type}, self.member.get_content())
        else:
            raise HTTPError(404, "Resource Not Found")
                    
    def DELETE(self):
        # delete resource, return 204
        
        if self.member:
            atom_store.remove_member(self.member)
        else:
            raise HTTPError(404, "Resource Not Found")

        return Response(204)        
